package com.yossi.kosherappstore

import android.content.Context
import android.content.pm.PackageManager
import android.graphics.BitmapFactory
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import androidx.core.view.isVisible
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import java.io.File

class AppAdapter(
    private val context: Context,
    private val installClickListener: OnInstallClickListener,
    private val uninstallClickListener: OnUninstallClickListener
) : ListAdapter<AppInfo, AppAdapter.AppViewHolder>(AppInfoDiffCallback()) {

    interface OnInstallClickListener {
        fun onInstallClick(appInfo: AppInfo)
    }

    interface OnUninstallClickListener {
        fun onUninstallClick(appInfo: AppInfo)
    }

    inner class AppViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        private val imageViewAppIcon: ImageView = itemView.findViewById(R.id.imageViewAppIcon)
        private val textViewAppName: TextView = itemView.findViewById(R.id.textViewAppName)
        private val textViewAppDescription: TextView = itemView.findViewById(R.id.textViewAppDescription)
        private val buttonInstall: Button = itemView.findViewById(R.id.buttonInstall)
        private val buttonUninstall: Button = itemView.findViewById(R.id.buttonUninstall)

        fun bind(appInfo: AppInfo) {
            Log.d("AppAdapter", "Binding ViewHolder for: ${appInfo.name}, Pkg: ${appInfo.packageName}")
            textViewAppName.text = appInfo.name
            textViewAppDescription.text = appInfo.description
            imageViewAppIcon.contentDescription = context.getString(R.string.app_icon_description_with_name, appInfo.name)


            appInfo.imageFileName?.let { fileName ->
                if (fileName.isNotEmpty()) {
                    val imgFile = File(context.filesDir, fileName)
                    if (imgFile.exists()) {
                        try {
                            val bitmap = BitmapFactory.decodeFile(imgFile.absolutePath)
                            imageViewAppIcon.setImageBitmap(bitmap)
                        } catch (e: OutOfMemoryError) {
                            Log.e("AppAdapter", "OutOfMemoryError loading image: ${imgFile.absolutePath}", e)
                            imageViewAppIcon.setImageResource(R.mipmap.ic_launcher) // Fallback
                        } catch (e: Exception) {
                            Log.e("AppAdapter", "Error loading image: ${imgFile.absolutePath}", e)
                            imageViewAppIcon.setImageResource(R.mipmap.ic_launcher) // Fallback
                        }
                    } else {
                        Log.w("AppAdapter", "Image file not found: ${imgFile.absolutePath}")
                        imageViewAppIcon.setImageResource(R.mipmap.ic_launcher)
                    }
                } else {
                    imageViewAppIcon.setImageResource(R.mipmap.ic_launcher)
                }
            } ?: run {
                imageViewAppIcon.setImageResource(R.mipmap.ic_launcher)
            }

            val appIsInstalled = isPackageInstalled(appInfo.packageName, context.packageManager)
            // Log.d("AppAdapter", "App: ${appInfo.name}, IsInstalled: $appIsInstalled") // אפשר להסיר אם פחות נחוץ

            buttonInstall.isVisible = !appIsInstalled
            buttonUninstall.isVisible = appIsInstalled

            buttonInstall.setOnClickListener {
                installClickListener.onInstallClick(appInfo)
            }
            buttonUninstall.setOnClickListener {
                uninstallClickListener.onUninstallClick(appInfo)
            }
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): AppViewHolder {
        val view = LayoutInflater.from(context).inflate(R.layout.list_item_app, parent, false)
        return AppViewHolder(view)
    }

    override fun onBindViewHolder(holder: AppViewHolder, position: Int) {
        val currentApp = getItem(position) // שימוש ב-getItem() של ListAdapter
        holder.bind(currentApp)
    }

    private fun isPackageInstalled(packageName: String, packageManager: PackageManager): Boolean {
        if (packageName.isBlank()) return false
        return try {
            packageManager.getPackageInfo(packageName, 0)
            true
        } catch (e: PackageManager.NameNotFoundException) {
            false
        }
    }
    // אין צורך ב-updateApps() ו-getItemCount() explicit.
    // השתמש ב-adapter.submitList(yourNewList) מ-MainActivity.
}

class AppInfoDiffCallback : DiffUtil.ItemCallback<AppInfo>() {
    override fun areItemsTheSame(oldItem: AppInfo, newItem: AppInfo): Boolean {
        // מזהה ייחודי - שם החבילה הוא בחירה טובה
        return oldItem.packageName == newItem.packageName
    }

    override fun areContentsTheSame(oldItem: AppInfo, newItem: AppInfo): Boolean {
        // data class מממש equals בצורה שמשווה את כל המאפיינים, וזה מה שאנחנו צריכים כאן.
        return oldItem == newItem
    }
}