package com.yossi.kosherappstore

import android.app.Activity
import android.content.ActivityNotFoundException
import android.content.ClipData
import android.content.ClipboardManager
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuItem
import android.widget.ProgressBar
import android.widget.TextView
import android.widget.Toast
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.FileProvider
import androidx.core.content.edit
import androidx.core.net.toUri
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import kotlinx.coroutines.*
import java.io.BufferedInputStream
import java.io.DataOutputStream
import java.io.File
import java.io.FileOutputStream
import java.io.IOException
import java.lang.reflect.Type
import java.net.URL

class MainActivity : AppCompatActivity(),
    AppAdapter.OnInstallClickListener,
    AppAdapter.OnUninstallClickListener {

    companion object {
        private const val TAG = "MainActivity_Locked" // או "MainActivity" לגרסה הרגילה
        const val PREFS_NAME = "KosherAppStorePrefs"
        const val KEY_APPS_LIST = "appsList"
        const val KEY_SETTINGS_PASSWORD = "settingsPassword" // נשאר למקרה שהגרסה הנעולה קוראת מ-prefs של גרסה רגילה
        const val DEFAULT_PASSWORD = "yossi6547" // נשאר
        const val KEY_INSTALL_LOCATION = "installLocation"
        const val KEY_USE_ROOT_INSTALL = "useRootInstall"
        const val KEY_NORMAL_INSTALL_MODE = "normalInstallMode"
        const val KEY_ROOT_INSTALL_METHOD = "rootInstallMethod"
        const val ROOT_METHOD_PM_INSTALL = "pm_install"
        const val ROOT_METHOD_COPY_TO_SYSTEM = "copy_to_system"
        const val KEY_ENABLE_SEND_FEEDBACK = "enableSendFeedback"
        const val KEY_ABOUT_TEXT = "aboutText"
        const val DEFAULT_ABOUT_TEXT = "זוהי חנות אפליקציות כשרה." // אפשר להתאים לגרסה
        const val FEEDBACK_EMAIL = "c51525355@gmail.com"
    }

    private lateinit var recyclerViewApps: RecyclerView
    private lateinit var appAdapter: AppAdapter
    private lateinit var sharedPreferences: SharedPreferences
    private val gson = Gson()

    private val activityScope = CoroutineScope(Dispatchers.Main + SupervisorJob())
    private var progressDialog: AlertDialog? = null
    private var downloadJob: Job? = null

    // settingsActivityResultLauncher יוסר מהגרסה הנעולה אם לא היה קיים

    private var appToUninstallPackageName: String? = null
    private val uninstallActivityResultLauncher =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {
            appToUninstallPackageName?.let { packageName ->
                try {
                    packageManager.getPackageInfo(packageName, 0)
                    Toast.makeText(this, "הסרת ההתקנה של $packageName בוטלה או נכשלה.", Toast.LENGTH_SHORT).show()
                } catch (e: PackageManager.NameNotFoundException) {
                    Toast.makeText(this, "$packageName הוסר בהצלחה.", Toast.LENGTH_SHORT).show()
                }
            }
            appToUninstallPackageName = null
            loadAndDisplayApps()
        }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        title = getString(R.string.app_name) // או כותרת ספציפית לגרסה הנעולה

        sharedPreferences = getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE)
        // אתחול ברירות מחדל (רלוונטי בעיקר אם זו הפעלה ראשונה של גרסה נעולה על מכשיר נקי)
        if (!sharedPreferences.contains(KEY_ROOT_INSTALL_METHOD)) {
            sharedPreferences.edit { putString(KEY_ROOT_INSTALL_METHOD, ROOT_METHOD_PM_INSTALL) }
        }
        if (!sharedPreferences.contains(KEY_INSTALL_LOCATION)) {
            sharedPreferences.edit { putString(KEY_INSTALL_LOCATION, "system/app") }
        }
        if (!sharedPreferences.contains(KEY_USE_ROOT_INSTALL)) {
            sharedPreferences.edit { putBoolean(KEY_USE_ROOT_INSTALL, true) }
        }
        if (!sharedPreferences.contains(KEY_NORMAL_INSTALL_MODE)) {
            sharedPreferences.edit { putBoolean(KEY_NORMAL_INSTALL_MODE, false) }
        }
        if (!sharedPreferences.contains(KEY_ABOUT_TEXT)) {
            sharedPreferences.edit { putString(KEY_ABOUT_TEXT, DEFAULT_ABOUT_TEXT) }
        }
        if (!sharedPreferences.contains(KEY_ENABLE_SEND_FEEDBACK)) {
            sharedPreferences.edit { putBoolean(KEY_ENABLE_SEND_FEEDBACK, true) }
        }

        recyclerViewApps = findViewById(R.id.recyclerViewApps)
        recyclerViewApps.layoutManager = LinearLayoutManager(this)
        appAdapter = AppAdapter(this, this, this) // context, installClickListener, uninstallClickListener
        recyclerViewApps.adapter = appAdapter

        Log.d(TAG, "onCreate: Calling loadAndDisplayApps for the first time")
        loadAndDisplayApps()
    }

    override fun onResume() {
        super.onResume()
        Log.d(TAG, "onResume: Calling loadAndDisplayApps")
        loadAndDisplayApps()
        invalidateOptionsMenu()
    }

    override fun onDestroy() {
        super.onDestroy()
        downloadJob?.cancel()
        activityScope.cancel()
        progressDialog?.dismiss()
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        menuInflater.inflate(R.menu.main_menu, menu) // ודא שזה התפריט הנכון (עם/בלי "הגדרות")
        return true
    }

    override fun onPrepareOptionsMenu(menu: Menu?): Boolean {
        // בגרסה הנעולה, פריט ההגדרות לא יהיה קיים ב-main_menu.xml
        val enableSendFeedback = sharedPreferences.getBoolean(KEY_ENABLE_SEND_FEEDBACK, true)
        menu?.findItem(R.id.action_send_feedback)?.isVisible = enableSendFeedback
        // אם יש פריט הגדרות בתפריט של הגרסה הרגילה:
        // menu?.findItem(R.id.action_settings)?.isVisible = true // או לפי לוגיקה אחרת
        return super.onPrepareOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            // אם זה פרויקט רגיל, הוסף case ל-R.id.action_settings
            // R.id.action_settings -> { showPasswordDialogForSettings(); true }
            R.id.action_send_feedback -> { sendFeedbackEmail(); true }
            R.id.action_about -> { startActivity(Intent(this, AboutActivity::class.java)); true }
            else -> super.onOptionsItemSelected(item)
        }
    }

    // הפונקציה showPasswordDialogForSettings() רלוונטית רק לגרסה הרגילה
    // private fun showPasswordDialogForSettings() { /* ... */ }

    private fun sendFeedbackEmail() {
        val emailSubject = "${getString(R.string.feedback_email_subject_prefix)} (${getString(R.string.app_name)})"
        val emailIntent = Intent(Intent.ACTION_SENDTO).apply {
            data = "mailto:".toUri()
            putExtra(Intent.EXTRA_EMAIL, arrayOf(FEEDBACK_EMAIL))
            putExtra(Intent.EXTRA_SUBJECT, emailSubject)
        }
        try {
            startActivity(Intent.createChooser(emailIntent, getString(R.string.feedback_email_chooser_title)))
            val clipboard = getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
            val clip = ClipData.newPlainText("כתובת מייל למשוב", FEEDBACK_EMAIL)
            clipboard.setPrimaryClip(clip)
            Toast.makeText(this, "${getString(R.string.feedback_email_copied_toast)}: $FEEDBACK_EMAIL", Toast.LENGTH_LONG).show()
        } catch (ex: ActivityNotFoundException) {
            Toast.makeText(this, getString(R.string.feedback_no_email_app_toast), Toast.LENGTH_SHORT).show()
        }
    }

    private fun loadAndDisplayApps() {
        Log.d(TAG, "loadAndDisplayApps: Starting to load apps from SharedPreferences.")
        val loadedApps = loadAppsListFromPrefs()
        Log.d(TAG, "loadAndDisplayApps: Loaded ${loadedApps.size} apps. Submitting list to adapter.")
        appAdapter.submitList(loadedApps.toMutableList())
    }

    private fun loadAppsListFromPrefs(): List<AppInfo> {
        val jsonApps = sharedPreferences.getString(KEY_APPS_LIST, null)
        return if (jsonApps != null) {
            val typeToken: Type = object : TypeToken<ArrayList<AppInfo>>() {}.type
            try {
                gson.fromJson(jsonApps, typeToken) ?: ArrayList()
            } catch (e: Exception) {
                Log.e(TAG, "Error parsing app list from prefs", e)
                ArrayList()
            }
        } else {
            ArrayList()
        }
    }

    override fun onInstallClick(appInfo: AppInfo) {
        Log.d(TAG, "onInstallClick for: ${appInfo.name}, URL: ${appInfo.apkUrl}, FileName: ${appInfo.apkFileName}")
        if (!appInfo.apkUrl.isNullOrBlank()) {
            downloadAndInstallApk(appInfo)
        } else {
            val apkFileInternal = File(filesDir, appInfo.apkFileName)
            if (appInfo.apkFileName.isNotBlank() && apkFileInternal.exists()) {
                proceedWithInstallation(apkFileInternal, appInfo)
            } else {
                Toast.makeText(this, "קובץ APK '${appInfo.apkFileName}' לא קיים מקומית ואין URL להורדה.", Toast.LENGTH_LONG).show()
                Log.e(TAG, "Local APK file '${appInfo.apkFileName}' not found and no URL for ${appInfo.name}")
            }
        }
    }

    private fun downloadAndInstallApk(appInfo: AppInfo) {
        val destinationFile = File(filesDir, appInfo.apkFileName)
        val dialogView = LayoutInflater.from(this).inflate(R.layout.dialog_progress, null)
        val progressBar = dialogView.findViewById<ProgressBar>(R.id.progressBarDownload)
        val progressText = dialogView.findViewById<TextView>(R.id.textViewProgress)

        progressDialog?.dismiss()
        progressDialog = AlertDialog.Builder(this)
            .setTitle("מוריד את ${appInfo.name}")
            .setView(dialogView)
            .setCancelable(false)
            .setNegativeButton("בטל הורדה") { _, _ ->
                downloadJob?.cancel("User cancelled download")
                Toast.makeText(this, "הורדה בוטלה", Toast.LENGTH_SHORT).show()
            }
            .create()
        progressDialog?.show()

        progressBar.isIndeterminate = true
        progressText.text = "מתחבר..."

        downloadJob = activityScope.launch(Dispatchers.IO) {
            var success = false
            var errorMessage: String? = "שגיאה לא ידועה"
            try {
                Log.d(TAG, "Starting download for ${appInfo.apkUrl} to ${destinationFile.absolutePath}")
                val url = URL(appInfo.apkUrl)
                val connection = url.openConnection().apply {
                    connectTimeout = 15000
                    readTimeout = 30000
                }
                connection.connect()

                val totalFileSize: Long = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                    connection.contentLengthLong
                } else {
                    val length = connection.contentLength
                    if (length == -1) -1L else length.toLong()
                }

                withContext(Dispatchers.Main) {
                    if (progressDialog?.isShowing != true) return@withContext
                    if (totalFileSize > 0) {
                        progressBar.isIndeterminate = false
                        progressBar.max = 100
                        progressText.text = getString(R.string.download_progress_percent, 0)
                    } else {
                        progressBar.isIndeterminate = true
                        progressText.text = "מוריד (גודל לא ידוע)..."
                    }
                }

                BufferedInputStream(connection.inputStream).use { input ->
                    FileOutputStream(destinationFile).use { output ->
                        val data = ByteArray(8192)
                        var totalBytesRead: Long = 0
                        var bytesReadThisTime: Int // הצהרה כאן

                        while (input.read(data).also { bytesReadThisTime = it } != -1 && isActive) { // אתחול בלולאה
                            totalBytesRead += bytesReadThisTime.toLong()
                            output.write(data, 0, bytesReadThisTime)
                            if (totalFileSize > 0) {
                                val progressPercentage = ((totalBytesRead * 100) / totalFileSize).toInt()
                                withContext(Dispatchers.Main) {
                                    if (progressDialog?.isShowing == true) {
                                        progressBar.progress = progressPercentage
                                        progressText.text = getString(R.string.download_progress_percent, progressPercentage)
                                    }
                                }
                            } else {
                                withContext(Dispatchers.Main) {
                                    if (progressDialog?.isShowing == true) {
                                        progressText.text = getString(R.string.download_progress_kb, totalBytesRead / 1024)
                                    }
                                }
                            }
                        }
                    }
                }

                if (isActive) {
                    Log.d(TAG, "Download completed for ${appInfo.apkUrl}")
                    success = true
                } else {
                    Log.d(TAG, "Download was cancelled during transfer for ${appInfo.apkUrl}")
                    // הקובץ יימחק ב-finally אם ההורדה בוטלה
                }

            } catch (e: CancellationException) {
                Log.d(TAG, "Download coroutine cancelled for ${appInfo.apkUrl}", e)
                errorMessage = "הורדה בוטלה"
            } catch (e: IOException) {
                errorMessage = "שגיאת רשת: ${e.localizedMessage}"
                Log.e(TAG, "IOException downloading APK from ${appInfo.apkUrl}", e)
            } catch (e: Exception) {
                errorMessage = e.localizedMessage ?: "שגיאה כללית לא צפויה"
                Log.e(TAG, "General error downloading APK from ${appInfo.apkUrl}", e)
            } finally {
                if (!success && destinationFile.exists()) { // אם ההורדה לא הצליחה והקובץ קיים
                    Log.d(TAG, "Download not successful or cancelled, deleting partial file: ${destinationFile.name}")
                    destinationFile.delete()
                }
            }

            withContext(Dispatchers.Main) {
                progressDialog?.dismiss()
                progressDialog = null
                if (success) {
                    proceedWithInstallation(destinationFile, appInfo)
                } else if (downloadJob?.isCancelled == false && isActive) {
                    Toast.makeText(applicationContext, "שגיאה בהורדת APK: $errorMessage", Toast.LENGTH_LONG).show()
                }
            }
        }
    }

    private fun proceedWithInstallation(apkFileToInstall: File, appInfo: AppInfo) {
        Toast.makeText(this, "ממשיך להתקנה של: ${appInfo.name}", Toast.LENGTH_SHORT).show()
        val useNormalInstall = sharedPreferences.getBoolean(KEY_NORMAL_INSTALL_MODE, false)
        val useRootGeneral = sharedPreferences.getBoolean(KEY_USE_ROOT_INSTALL, true)

        if (useNormalInstall) {
            Log.d(TAG, "Attempting normal installation for ${appInfo.name}")
            installApkNormally(apkFileToInstall)
        } else if (useRootGeneral) {
            Log.d(TAG, "Attempting root installation for ${appInfo.name}")
            val rootInstallMethod = sharedPreferences.getString(KEY_ROOT_INSTALL_METHOD, ROOT_METHOD_PM_INSTALL)
            if (rootInstallMethod == ROOT_METHOD_PM_INSTALL) {
                Log.d(TAG, "Using PM Install method.")
                executePmInstallRootCommand(apkFileToInstall.absolutePath)
            } else {
                Log.d(TAG, "Using Copy to System method.")
                val installPathPref = sharedPreferences.getString(KEY_INSTALL_LOCATION, "system/app")
                val targetDirName = appInfo.packageName
                if (targetDirName.isBlank()) {
                    Toast.makeText(this, "שם החבילה חסר עבור ${appInfo.name}", Toast.LENGTH_LONG).show()
                    return
                }
                val targetPath = "/$installPathPref/$targetDirName"
                executeCopyToSystemRootCommands(apkFileToInstall.absolutePath, targetPath, appInfo.apkFileName)
            }
        } else {
            Toast.makeText(this, "אף שיטת התקנה לא הופעלה.", Toast.LENGTH_LONG).show()
        }
        // רענון ב-onResume יטפל בעדכון ה-UI
    }

    override fun onUninstallClick(appInfo: AppInfo) {
        AlertDialog.Builder(this)
            .setTitle("הסר התקנה")
            .setMessage("האם אתה בטוח שברצונך להסיר את ההתקנה של '${appInfo.name}' מהמכשיר?")
            .setPositiveButton(getString(R.string.uninstall_button_text)) { dialog, _ ->
                performUninstallAppFromDevice(appInfo)
                dialog.dismiss()
            }
            .setNegativeButton("ביטול", null)
            .show()
    }

    private fun performUninstallAppFromDevice(appInfo: AppInfo) {
        try {
            packageManager.getPackageInfo(appInfo.packageName, 0)
        } catch (e: PackageManager.NameNotFoundException) {
            Toast.makeText(this, "'${appInfo.name}' כבר לא מותקנת.", Toast.LENGTH_SHORT).show()
            loadAndDisplayApps()
            return
        }

        val wasInstalledToSystemByThisApp =
            !sharedPreferences.getBoolean(KEY_NORMAL_INSTALL_MODE, false) &&
                    sharedPreferences.getBoolean(KEY_USE_ROOT_INSTALL, true) &&
                    sharedPreferences.getString(KEY_ROOT_INSTALL_METHOD, ROOT_METHOD_PM_INSTALL) == ROOT_METHOD_COPY_TO_SYSTEM

        if (wasInstalledToSystemByThisApp) {
            Toast.makeText(this, "מנסה להסיר את '${appInfo.name}' כאפליקציית מערכת...", Toast.LENGTH_LONG).show()
            val installPathPref = sharedPreferences.getString(KEY_INSTALL_LOCATION, "system/app")
            val targetDirPath = "/$installPathPref/${appInfo.packageName}"
            executeRemoveSystemAppRootCommands(targetDirPath, appInfo.packageName)
        } else {
            appToUninstallPackageName = appInfo.packageName
            val uninstallIntent = Intent(Intent.ACTION_DELETE).apply {
                data = "package:${appInfo.packageName}".toUri()
                putExtra(Intent.EXTRA_RETURN_RESULT, true)
            }
            try {
                Log.d(TAG, "Attempting to uninstall via Intent: ${appInfo.packageName}")
                uninstallActivityResultLauncher.launch(uninstallIntent)
            } catch (e: Exception) {
                Toast.makeText(this, "שגיאה בניסיון הסרת התקנה: ${e.message}", Toast.LENGTH_LONG).show()
                Log.e(TAG, "Error during normal uninstall intent for ${appInfo.packageName}: ${e.message}")
                appToUninstallPackageName = null
            }
        }
    }

    private fun installApkNormally(apkFile: File) {
        try {
            val apkUri = FileProvider.getUriForFile(this, "${applicationContext.packageName}.provider", apkFile)
            val installIntent = Intent(Intent.ACTION_VIEW).apply {
                setDataAndType(apkUri, "application/vnd.android.package-archive")
                addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
            }
            Log.d(TAG, "Starting normal install intent for URI: $apkUri")
            startActivity(installIntent)
        } catch (e: Exception) {
            Toast.makeText(this, "שגיאה בניסיון להתקין APK: ${e.message}", Toast.LENGTH_LONG).show()
            Log.e(TAG, "Error attempting to install APK: ${e.message}")
        }
    }

    private fun executeCopyToSystemRootCommands(sourceApkPath: String, targetDirPath: String, targetApkName: String) {
        activityScope.launch(Dispatchers.IO) {
            var success = false
            try {
                Log.d(TAG, "Attempting Copy to System SU commands for $targetApkName")
                val suProcess = Runtime.getRuntime().exec("su")
                DataOutputStream(suProcess.outputStream).use { os ->
                    os.writeBytes("mount -o rw,remount /system\n")
                    os.writeBytes("mkdir -p '$targetDirPath'\n")
                    os.writeBytes("cp '$sourceApkPath' '$targetDirPath/$targetApkName'\n")
                    os.writeBytes("chmod 644 '$targetDirPath/$targetApkName'\n")
                    os.writeBytes("chown root:root '$targetDirPath/$targetApkName'\n")
                    os.writeBytes("chmod 755 '$targetDirPath'\n")
                    os.writeBytes("chown root:root '$targetDirPath'\n")
                    os.writeBytes("mount -o ro,remount /system\n")
                    os.writeBytes("exit\n")
                    os.flush()
                }
                val exitValue = suProcess.waitFor()
                if (exitValue == 0) success = true
                withContext(Dispatchers.Main) {
                    if (success) {
                        Toast.makeText(this@MainActivity, "העתקה למערכת בוצעה (ייתכן שיידרש ריבוט)", Toast.LENGTH_SHORT).show()
                    } else {
                        Toast.makeText(this@MainActivity, "שגיאה בהעתקה למערכת, קוד יציאה: $exitValue", Toast.LENGTH_LONG).show()
                    }
                    loadAndDisplayApps()
                }
                suProcess.destroy()
            } catch (e: Exception) {
                Log.e(TAG, "Exception in executeCopyToSystemRootCommands", e)
                withContext(Dispatchers.Main) { Toast.makeText(this@MainActivity, "שגיאה: ${e.message}", Toast.LENGTH_LONG).show() }
            }
        }
    }

    private fun executePmInstallRootCommand(apkPath: String) {
        activityScope.launch(Dispatchers.IO) {
            var success = false
            try {
                Log.d(TAG, "Attempting PM Install SU command for $apkPath")
                val suProcess = Runtime.getRuntime().exec("su")
                DataOutputStream(suProcess.outputStream).use { os ->
                    os.writeBytes("pm install -r -g \"$apkPath\"\n"); os.writeBytes("exit\n"); os.flush()
                }
                val exitValue = suProcess.waitFor()
                if (exitValue == 0) success = true
                withContext(Dispatchers.Main) {
                    if (success) {
                        Toast.makeText(this@MainActivity, "הותקן (pm install)", Toast.LENGTH_SHORT).show()
                    } else {
                        Toast.makeText(this@MainActivity, "שגיאה (pm install), קוד: $exitValue", Toast.LENGTH_LONG).show()
                        val error = suProcess.errorStream.bufferedReader().readText()
                        if(error.isNotBlank()) Log.e(TAG, "PM Install Error: $error")
                    }
                    loadAndDisplayApps()
                }
                suProcess.destroy()
            } catch (e: Exception) {
                Log.e(TAG, "Exception in executePmInstallRootCommand", e)
                withContext(Dispatchers.Main) { Toast.makeText(this@MainActivity, "שגיאה: ${e.message}", Toast.LENGTH_LONG).show() }
            }
        }
    }

    private fun executeRemoveSystemAppRootCommands(targetDirPath: String, packageName: String) {
        activityScope.launch(Dispatchers.IO) {
            var success = false
            try {
                Log.d(TAG, "Attempting Remove System App SU for $packageName")
                val suProcess = Runtime.getRuntime().exec("su")
                DataOutputStream(suProcess.outputStream).use { os ->
                    os.writeBytes("mount -o rw,remount /system\n")
                    os.writeBytes("rm -rf '$targetDirPath'\n")
                    os.writeBytes("pm uninstall \"$packageName\"\n")
                    os.writeBytes("mount -o ro,remount /system\n")
                    os.writeBytes("exit\n"); os.flush()
                }
                val exitValue = suProcess.waitFor()
                if (exitValue == 0) success = true
                withContext(Dispatchers.Main) {
                    if (success) {
                        Toast.makeText(this@MainActivity, "'$packageName' הוסר (ייתכן שיידרש ריבוט)", Toast.LENGTH_LONG).show()
                    } else {
                        Toast.makeText(this@MainActivity, "שגיאה בהסרת '$packageName', קוד: $exitValue", Toast.LENGTH_LONG).show()
                        val error = suProcess.errorStream.bufferedReader().readText()
                        if(error.isNotBlank()) Log.e(TAG, "Remove System App Error: $error")
                    }
                    loadAndDisplayApps()
                }
                suProcess.destroy()
            } catch (e: Exception) {
                Log.e(TAG, "Exception in executeRemoveSystemAppRootCommands", e)
                withContext(Dispatchers.Main) { Toast.makeText(this@MainActivity, "שגיאה: ${e.message}", Toast.LENGTH_LONG).show() }
            }
        }
    }
}